//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                           DateTimeInstance.mqh |
//|                                                                                                                                                                            hoz |
//|                                                                                                                                                                                |
//+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
#property copyright "hoz"
#property link      ""
#property version   "1.00"

#define SECONDS   0
#define MINUTES   1
#define HOURS     2
#define DAYS      3
#define WEEKS     4
#define MONTHS    5

//+================================================================================================================================================================================+
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//|                                                                        КЛАСС ДЛЯ РАБОТЫ С ДАТОЙ И ВРЕМЕНЕМ                                                                     |
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+================================================================================================================================================================================+
class DateTimeInstance {
  private:

  public:
    MqlDateTime m_dtStructure;
    // Переменные, хранящие количество секунд в соответствующем таймфрейме
    static const int m1;
    static const int m5;
    static const int m15;
    static const int m30;
    static const int h1;
    static const int h4;
    static const int d1;
    static const int w1;
    static const int mn1;
  
    //---- Конструкторы
    DateTimeInstance();
   ~DateTimeInstance();
    DateTimeInstance(const datetime dt);
    DateTimeInstance(const datetime dt, const int year, const int month, const int day);
//    DateTimeInstance(const datetime dt, const int hour, const int minute, const int second);
    DateTimeInstance(const DateTimeInstance& instance);
    DateTimeInstance(const DateTimeInstance& instance, const int hour, const int minute, const int second);

    //---- Методы
    DateTimeInstance *getDateTimeInstance();
    datetime minus(const int multiplier, const int tf = SECONDS);
    string dayOfWeek();

    //--- Переопределение операторов
    void operator=(const DateTimeInstance& dt);
    bool operator==(const DateTimeInstance& dt);
    bool operator!=(const DateTimeInstance& dt);
    bool operator<(const DateTimeInstance& dt);
    bool operator>(const DateTimeInstance& dt);
    bool operator<=(const DateTimeInstance& dt);
    bool operator>=(const DateTimeInstance& dt);
};

const int DateTimeInstance :: m1  =  60;
const int DateTimeInstance :: m5  =  300;
const int DateTimeInstance :: m15 =  900;
const int DateTimeInstance :: m30 =  1800;
const int DateTimeInstance :: h1  =  3600;
const int DateTimeInstance :: h4  =  14400;
const int DateTimeInstance :: d1  =  86400;
const int DateTimeInstance :: w1  =  604800;
const int DateTimeInstance :: mn1 =  2419200;

//==================================================================================================================================================================================
// Конструктор класса. =============================================================================================================================================================
DateTimeInstance :: DateTimeInstance() {
}
//==================================================================================================================================================================================
// Конструкторы класса. ============================================================================================================================================================
DateTimeInstance :: DateTimeInstance(const datetime dt) {
  TimeToStruct(dt, m_dtStructure);
}

DateTimeInstance :: DateTimeInstance(const datetime dt,    // Значение типа datetime
                                     const int year,       // Год
                                     const int month,      // Месяц
                                     const int day) {      // День
  TimeToStruct(dt, m_dtStructure);
  m_dtStructure.year = year;
  m_dtStructure.mon = month;
  m_dtStructure.day = day;
}

/*DateTimeInstance :: DateTimeInstance(const datetime dt,     // Значение типа datetime
                                     const int hour,        // Количество часов
                                     const int minute,      // Количество минут
                                     const int second) {    // Количество секунд
  TimeToStruct(dt, m_dtStructure);
  m_dtStructure.hour = hour;
  m_dtStructure.min = minute;
  m_dtStructure.sec = second;
}*/

DateTimeInstance :: DateTimeInstance(const DateTimeInstance& instance) {
  m_dtStructure.year = instance.m_dtStructure.year;
  m_dtStructure.mon = instance.m_dtStructure.mon;
  m_dtStructure.day = instance.m_dtStructure.day;
  m_dtStructure.hour = instance.m_dtStructure.hour;
  m_dtStructure.min = instance.m_dtStructure.min;
  m_dtStructure.sec = instance.m_dtStructure.sec;
}

DateTimeInstance :: DateTimeInstance(const DateTimeInstance& instance, const int hour, const int minute, const int second) {
  m_dtStructure.year = instance.m_dtStructure.year;
  m_dtStructure.mon = instance.m_dtStructure.mon;
  m_dtStructure.day = instance.m_dtStructure.day;
  m_dtStructure.hour = instance.m_dtStructure.hour + hour;
  m_dtStructure.min = instance.m_dtStructure.min + minute;
  m_dtStructure.sec = instance.m_dtStructure.sec + second;
}
//==================================================================================================================================================================================
// Деструктор класса. ==============================================================================================================================================================
DateTimeInstance :: ~DateTimeInstance() {
}
//==================================================================================================================================================================================
// Переопределение оператора `=`. ==================================================================================================================================================
void DateTimeInstance :: operator=(const DateTimeInstance& dt) {
//---
  m_dtStructure.year = dt.m_dtStructure.year;
  m_dtStructure.mon = dt.m_dtStructure.mon;
  m_dtStructure.day = dt.m_dtStructure.day;
  m_dtStructure.hour = dt.m_dtStructure.hour;
  m_dtStructure.min = dt.m_dtStructure.min;
  m_dtStructure.sec = dt.m_dtStructure.sec;
}
//==================================================================================================================================================================================
// Переопределение оператора `==`. =================================================================================================================================================
bool DateTimeInstance :: operator==(const DateTimeInstance& dt) {
//---
  if (m_dtStructure.year != dt.m_dtStructure.year)
    return false;
  if (m_dtStructure.mon != dt.m_dtStructure.mon)
    return false;
  if (m_dtStructure.day != dt.m_dtStructure.day)
    return false;
//  if (m_dtStructure.hour != dt.m_dtStructure.hour)
  //  return false;
//  if (m_dtStructure.min != dt.m_dtStructure.min)
  //  return false;
//  if (m_dtStructure.sec != dt.m_dtStructure.sec)
  //  return false;
//---
  return true;
}
//==================================================================================================================================================================================
// Переопределение оператора `!=`. =================================================================================================================================================
bool DateTimeInstance :: operator!=(const DateTimeInstance& dt) {
//---
  if (m_dtStructure.year != dt.m_dtStructure.year)
    return true;
  if (m_dtStructure.mon != dt.m_dtStructure.mon)
    return true;
  if (m_dtStructure.day != dt.m_dtStructure.day)
    return true;
//  if (m_dtStructure.hour != dt.m_dtStructure.hour)
//    return true;
//  if (m_dtStructure.min != dt.m_dtStructure.min)
  //  return true;
//  if (m_dtStructure.sec != dt.m_dtStructure.sec)
  //  return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `<`. ==================================================================================================================================================
bool DateTimeInstance :: operator<(const DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) < StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `>`. ==================================================================================================================================================
bool DateTimeInstance :: operator>(const DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) > StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `<=`. ==================================================================================================================================================
bool DateTimeInstance :: operator<=(const DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) <= StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Переопределение оператора `>`. ==================================================================================================================================================
bool DateTimeInstance :: operator>=(const DateTimeInstance& dt) {
//---
  if (StructToTime(m_dtStructure) >= StructToTime(dt.m_dtStructure))
    return true;
//---
  return false;
}
//==================================================================================================================================================================================
// Возвращает указатель на объект собственного класса. =============================================================================================================================
DateTimeInstance *DateTimeInstance :: getDateTimeInstance(void) {
  return GetPointer(this);
}
//==================================================================================================================================================================================
// Возвращает результат вычета из даты объекта, на котором вызывается метод, заданного количества времени. =========================================================================
datetime DateTimeInstance :: minus(const int multiplier, const int tf = SECONDS) {
  switch(tf) {
    case SECONDS:
      return StructToTime(this.m_dtStructure) - multiplier * 1;
    case MINUTES:
      return StructToTime(this.m_dtStructure) - multiplier * m1;
    case HOURS:
      return StructToTime(this.m_dtStructure) - multiplier * h1;
    case DAYS:
      return StructToTime(this.m_dtStructure) - multiplier * d1;
    case WEEKS:
      return StructToTime(this.m_dtStructure) - multiplier * w1;
    case MONTHS:
      return StructToTime(this.m_dtStructure) - multiplier * mn1;
    default:
      return StructToTime(this.m_dtStructure) - multiplier * 1;
  }
}
//==================================================================================================================================================================================
// Возвращает день недели. =========================================================================================================================================================
string DateTimeInstance :: dayOfWeek() {
  switch (m_dtStructure.day_of_week) {
    case MONDAY:
      return "Понедельник";
    case TUESDAY:
      return "Вторник";
    case WEDNESDAY:
      return "Среда";
    case THURSDAY:
      return "Четверг";
    case FRIDAY:
      return "Пятница";
    case SATURDAY:
      return "Суббота";
    case SUNDAY:
      return "Воскресенье";
  }
  return "";
}